@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 # Copyright 2011 Joe Wreschnig, Christoph Reiter
+#           2016 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -8,13 +9,14 @@
 import os
 import sys
 import bz2
-import urllib2
-import urllib
 import itertools
 
 from gi.repository import Gtk, GLib, Pango
 
+from quodlibet.util.dprint import print_d
+
 import quodlibet
+from quodlibet import _
 from quodlibet import qltk
 from quodlibet import util
 from quodlibet import config
@@ -24,22 +26,28 @@ from quodlibet.formats.remote import RemoteFile
 from quodlibet.formats._audio import TAG_TO_SORT, MIGRATE, AudioFile
 from quodlibet.library import SongLibrary
 from quodlibet.query import Query
+from quodlibet.compat import reduce, urlopen
 from quodlibet.qltk.getstring import GetStringDialog
 from quodlibet.qltk.songsmenu import SongsMenu
 from quodlibet.qltk.notif import Task
-from quodlibet.qltk import Icons
-from quodlibet.util import copool, connect_destroy, sanitize_tags, connect_obj
+from quodlibet.qltk import Icons, ErrorMessage, WarningMessage
+from quodlibet.util import copool, connect_destroy, sanitize_tags, \
+    connect_obj, escape
+from quodlibet.util.i18n import numeric_phrase
+from quodlibet.util.path import uri_is_valid
 from quodlibet.util.string import decode, encode
-from quodlibet.util.uri import URI
+from quodlibet.util import print_w
 from quodlibet.qltk.views import AllTreeView
 from quodlibet.qltk.searchbar import SearchBarBox
 from quodlibet.qltk.completion import LibraryTagCompletion
 from quodlibet.qltk.x import MenuItem, Align, ScrolledWindow
 from quodlibet.qltk.x import SymbolicIconImage
 from quodlibet.qltk.menubutton import MenuButton
+from quodlibet.compat import text_type, iteritems, iterkeys
+
 
 STATION_LIST_URL = \
-    "http://bitbucket.org/lazka/quodlibet/downloads/radiolist.bz2"
+    "https://bitbucket.org/lazka/quodlibet/downloads/radiolist.bz2"
 STATIONS_FAV = os.path.join(quodlibet.get_user_dir(), "stations")
 STATIONS_ALL = os.path.join(quodlibet.get_user_dir(), "stations_all")
 
@@ -106,8 +114,8 @@ class IRFile(RemoteFile):
         for tag in ["title", "artist", "~format"]:
             value = self.get(tag)
             if value is not None:
-                lines.append("%s=%s" % (tag, encode(value)))
-        return "\n".join(lines)
+                lines.append(encode(tag) + b"=" + encode(value))
+        return b"\n".join(lines)
 
     def can_change(self, k=None):
         if self.streamsong:
@@ -166,7 +174,7 @@ def ParsePLS(file):
         count += 1
 
     if warnings:
-        qltk.WarningMessage(
+        WarningMessage(
             None, _("Unsupported file type"),
             _("Station lists can only contain locations of stations, "
               "not other station lists or playlists. The following locations "
@@ -197,23 +205,28 @@ def ParseM3U(fileobj):
 
 
 def add_station(uri):
-    """Fetches the URI content and extracts IRFiles"""
+    """Fetches the URI content and extracts IRFiles
+    Returns None in error, else a possibly filled list of stations"""
 
     irfs = []
-    if isinstance(uri, unicode):
+    if isinstance(uri, text_type):
         uri = uri.encode('utf-8')
 
     if uri.lower().endswith(".pls") or uri.lower().endswith(".m3u"):
         try:
-            sock = urllib.urlopen(uri)
+            sock = urlopen(uri)
         except EnvironmentError as e:
+            print_d("Got %s from %s" % (uri, e))
             encoding = util.get_locale_encoding()
             try:
                 err = e.strerror.decode(encoding, 'replace')
-            except (TypeError, AttributeError):
+            except TypeError:
                 err = e.strerror[1].decode(encoding, 'replace')
-            qltk.ErrorMessage(None, _("Unable to add station"), err).run()
-            return []
+            except AttributeError:
+                # Give up and display the exception - may be useful HTTP info
+                err = str(e)
+            ErrorMessage(None, _("Unable to add station"), escape(err)).run()
+            return None
 
         if uri.lower().endswith(".pls"):
             irfs = ParsePLS(sock)
@@ -224,8 +237,8 @@ def add_station(uri):
     else:
         try:
             irfs = [IRFile(uri)]
-        except ValueError, err:
-            qltk.ErrorMessage(None, _("Unable to add station"), err).run()
+        except ValueError as err:
+            ErrorMessage(None, _("Unable to add station"), err).run()
 
     return irfs
 
@@ -241,8 +254,8 @@ def download_taglist(callback, cofuncid, step=1024 * 10):
             task.copool(cofuncid)
 
         try:
-            response = urllib2.urlopen(STATION_LIST_URL)
-        except urllib2.URLError:
+            response = urlopen(STATION_LIST_URL)
+        except EnvironmentError:
             GLib.idle_add(callback, None)
             return
 
@@ -341,11 +354,7 @@ class AddNewStation(GetStringDialog):
         for line in text.splitlines():
             line = line.strip()
 
-            try:
-                URI(line)
-            except ValueError:
-                pass
-            else:
+            if uri_is_valid(line):
                 return line
 
 
@@ -453,18 +462,7 @@ class QuestionBar(Gtk.InfoBar):
         content.add(label)
 
         self.add_button(_("_Load Stations"), self.RESPONSE_LOAD)
-
-        try:
-            self.set_show_close_button(True)
-        except AttributeError:
-            # < gtk3.10
-            action = self.get_action_area()
-            close_button = CloseButton()
-            close_button.show()
-            self.add_action_widget(close_button, Gtk.ResponseType.CLOSE)
-            # FIXME: doesn't seem to have any effect
-            action.set_child_packing(
-                close_button, False, True, 2, Gtk.PackType.START)
+        self.set_show_close_button(True)
 
     def __response(self, bar, response_id):
         if response_id == Gtk.ResponseType.CLOSE:
@@ -693,7 +691,7 @@ class InternetRadio(Browser, util.InstanceTracker):
 
         # keep at most 2 URLs for each group
         stations = []
-        for key, sub in groups.iteritems():
+        for key, sub in iteritems(groups):
             sub.sort(key=lambda s: s.get("~#listenerpeak", 0), reverse=True)
             stations.extend(sub[:2])
 
@@ -710,11 +708,11 @@ class InternetRadio(Browser, util.InstanceTracker):
         # update the libraries
         stations = dict(((s.key, s) for s in stations))
         # don't add ones that are in the fav list
-        for fav in self.__fav_stations.iterkeys():
+        for fav in iterkeys(self.__fav_stations):
             stations.pop(fav, None)
 
         # separate
-        o, n = set(self.__stations.iterkeys()), set(stations)
+        o, n = set(iterkeys(self.__stations)), set(stations)
         to_add, to_change, to_remove = n - o, o & n, o - n
         del o, n
 
@@ -725,7 +723,7 @@ class InternetRadio(Browser, util.InstanceTracker):
             # clear everything except stats
             AudioFile.reload(old)
             # add new metadata except stats
-            for k in (x for x in new.iterkeys() if x not in MIGRATE):
+            for k in (x for x in iterkeys(new) if x not in MIGRATE):
                 old[k] = new[k]
 
         to_add = [stations.pop(k) for k in to_add]
@@ -805,17 +803,19 @@ class InternetRadio(Browser, util.InstanceTracker):
 
     def __add_station(self, uri):
         irfs = add_station(uri)
-
+        if irfs is None:
+            # Error, rather than empty list
+            return
         if not irfs:
-            qltk.ErrorMessage(
+            ErrorMessage(
                 None, _("No stations found"),
                 _("No Internet radio stations were found at %s.") %
                 util.escape(uri)).run()
             return
 
-        irfs = filter(lambda station: station not in self.__fav_stations, irfs)
+        irfs = set(irfs) - set(self.__fav_stations)
         if not irfs:
-            qltk.WarningMessage(
+            WarningMessage(
                 None, _("Unable to add station"),
                 _("All stations listed are already in your library.")).run()
 
@@ -849,7 +849,7 @@ class InternetRadio(Browser, util.InstanceTracker):
         return menu
 
     def restore(self):
-        text = config.get("browsers", "query_text").decode("utf-8")
+        text = config.gettext("browsers", "query_text")
         self.__searchbar.set_text(text)
         if Query.is_parsable(text):
             self.__filter_changed(self.__searchbar, text, restore=True)
@@ -911,8 +911,8 @@ class InternetRadio(Browser, util.InstanceTracker):
         return True
 
     def save(self):
-        text = self.__searchbar.get_text().encode("utf-8")
-        config.set("browsers", "query_text", text)
+        text = self.__searchbar.get_text()
+        config.settext("browsers", "query_text", text)
 
         selection = self.view.get_selection()
         model, rows = selection.get_selected_rows()
@@ -937,8 +937,9 @@ class InternetRadio(Browser, util.InstanceTracker):
         self.view.set_cursor(path)
         self.view.scroll_to_cell(path, use_align=True, row_align=0.5)
 
-    def statusbar(self, i):
-        return ngettext("%(count)d station", "%(count)d stations", i)
+    def status_text(self, count, time=None):
+        return numeric_phrase("%(count)d station", "%(count)d stations",
+                              count, 'count')
 
 
 from quodlibet import app
