@@ -7,12 +7,12 @@
 # published by the Free Software Foundation
 
 import os
-import urllib
 from tempfile import NamedTemporaryFile
 
 from gi.repository import Gtk, GLib, Pango, Gdk
 
 import quodlibet
+from quodlibet import _
 from quodlibet import config
 from quodlibet import qltk
 from quodlibet.browsers import Browser
@@ -35,6 +35,7 @@ from quodlibet.util import connect_obj
 from quodlibet.util.path import get_home_dir
 from quodlibet.util.dprint import print_d, print_w
 from quodlibet.util.collection import FileBackedPlaylist
+from quodlibet.util.urllib import urlopen
 
 from .util import parse_m3u, parse_pls, PLAYLISTS, ConfirmRemovePlaylistDialog
 
@@ -70,6 +71,7 @@ class PlaylistsBrowser(Browser, DisplayPatternMixin):
 
     @classmethod
     def init(klass, library):
+        klass.library = library
         model = klass.__lists.get_model()
         for playlist in os.listdir(PLAYLISTS):
             try:
@@ -106,6 +108,7 @@ class PlaylistsBrowser(Browser, DisplayPatternMixin):
         for row in model:
             if row[0] is playlist:
                 if refresh:
+                    print_d("Refreshing playlist %s..." % row[0])
                     klass.__lists.row_changed(row.path, row.iter)
                 playlist.write()
                 break
@@ -117,21 +120,21 @@ class PlaylistsBrowser(Browser, DisplayPatternMixin):
     def __removed(klass, library, songs):
         for playlist in klass.playlists():
             if playlist.remove_songs(songs):
-                PlaylistsBrowser.changed(playlist)
+                klass.changed(playlist)
 
     @classmethod
     def __added(klass, library, songs):
         filenames = {song("~filename") for song in songs}
         for playlist in klass.playlists():
             if playlist.add_songs(filenames, library):
-                PlaylistsBrowser.changed(playlist)
+                klass.changed(playlist)
 
     @classmethod
     def __changed(klass, library, songs):
         for playlist in klass.playlists():
             for song in songs:
                 if song in playlist.songs:
-                    PlaylistsBrowser.changed(playlist, refresh=False)
+                    klass.changed(playlist)
                     break
 
     def cell_data(self, col, cell, model, iter, data):
@@ -164,6 +167,7 @@ class PlaylistsBrowser(Browser, DisplayPatternMixin):
     __lists.set_default_sort_func(ObjectStore._sort_on_value)
 
     def __init__(self, library):
+        self.library = library
         super(PlaylistsBrowser, self).__init__(spacing=6)
         self.set_orientation(Gtk.Orientation.VERTICAL)
         self.__render = self.__create_cell_renderer()
@@ -326,24 +330,27 @@ class PlaylistsBrowser(Browser, DisplayPatternMixin):
         model, iter = self.__view.get_selection().get_selected()
         if iter:
             playlist = model[iter][0]
+            removals = [song_at(iter_remove) for iter_remove in iters]
             if self._query is None or not self.get_filter_text():
                 # Calling playlist.remove_songs(songs) won't remove the
                 # right ones if there are duplicates
                 remove_from_model(iters, smodel)
                 self.__rebuild_playlist_from_songs_model(playlist, smodel)
+                # Emit manually
+                self.library.emit('changed', removals)
             else:
-                removals = [song_at(iter_remove) for iter_remove in iters]
                 print_d("Removing %d song(s) from %s"
                         % (len(removals), playlist))
                 playlist.remove_songs(removals, True)
                 remove_from_model(iters, smodel)
-            PlaylistsBrowser.changed(playlist)
+            self.changed(playlist)
             self.activate()
 
-    @classmethod
-    def __rebuild_playlist_from_songs_model(cls, playlist, smodel):
+    def __rebuild_playlist_from_songs_model(self, playlist, smodel):
+        playlist.inhibit = True
         playlist.clear()
         playlist.extend([row[0] for row in smodel])
+        playlist.inhibit = False
 
     def __drag_data_received(self, view, ctx, x, y, sel, tid, etime, library):
         # TreeModelSort doesn't support GtkTreeDragDestDrop.
@@ -364,7 +371,7 @@ class PlaylistsBrowser(Browser, DisplayPatternMixin):
             else:
                 playlist = model[path][0]
                 playlist.extend(songs)
-            PlaylistsBrowser.changed(playlist)
+            self.changed(playlist)
             Gtk.drag_finish(ctx, True, False, etime)
         else:
             if tid == DND_URI_LIST:
@@ -379,7 +386,7 @@ class PlaylistsBrowser(Browser, DisplayPatternMixin):
             name = name or os.path.basename(uri) or _("New Playlist")
             uri = uri.encode('utf-8')
             try:
-                sock = urllib.urlopen(uri)
+                sock = urlopen(uri)
                 f = NamedTemporaryFile()
                 f.write(sock.read())
                 f.flush()
@@ -392,7 +399,7 @@ class PlaylistsBrowser(Browser, DisplayPatternMixin):
                 library.add_filename(playlist)
                 if name:
                     playlist.rename(name)
-                PlaylistsBrowser.changed(playlist)
+                self.changed(playlist)
                 Gtk.drag_finish(ctx, True, False, etime)
             except IOError:
                 Gtk.drag_finish(ctx, False, False, etime)
@@ -577,7 +584,7 @@ class PlaylistsBrowser(Browser, DisplayPatternMixin):
                     _("Quod Libet can only import playlists in the M3U "
                       "and PLS formats.")).run()
                 return
-            PlaylistsBrowser.changed(playlist)
+            self.changed(playlist)
             library.add(playlist)
 
     def restore(self):
@@ -609,7 +616,7 @@ class PlaylistsBrowser(Browser, DisplayPatternMixin):
             playlist = FileBackedPlaylist.from_songs(PLAYLISTS, songs)
             GLib.idle_add(self._select_playlist, playlist)
         if playlist:
-            PlaylistsBrowser.changed(playlist, refresh=False)
+            self.changed(playlist, refresh=False)
 
 
 class PreferencesButton(Gtk.HBox):
