@@ -9,13 +9,14 @@
 
 import os
 
+from senf import uri2fsn, fsnative, fsn2text
+
 from quodlibet.util.string import split_escape
 
 from quodlibet import browsers
 
 from quodlibet.compat import cBytesIO as StringIO
 from quodlibet import util
-from quodlibet.util.path import fsnative, uri_to_path
 from quodlibet.util import print_d, print_e
 
 from quodlibet.qltk.browser import LibraryBrowser
@@ -157,23 +158,6 @@ def _volume(app, value):
     app.player.volume = min(1.0, max(0.0, volume))
 
 
-@registry.register("order", args=1)
-def _order(app, value):
-    order = app.window.order
-
-    if value in ["t", "toggle"]:
-        order.set_shuffle(not order.get_shuffle())
-        return
-
-    try:
-        order.set_active_by_name(value.lower())
-    except ValueError:
-        try:
-            order.set_active_by_index(int(value))
-        except (ValueError, IndexError):
-            pass
-
-
 @registry.register("stop-after", args=1)
 def _stop_after(app, value):
     po = app.player_options
@@ -187,15 +171,27 @@ def _stop_after(app, value):
         raise CommandError("Invalid value %r" % value)
 
 
+@registry.register("shuffle", args=1)
+def _shuffle(app, value):
+    po = app.player_options
+    if value in ["0", "off"]:
+        po.shuffle = False
+    elif value in ["1", "on"]:
+        po.shuffle = True
+    elif value in ["t", "toggle"]:
+        po.shuffle = not po.shuffle
+
+
 @registry.register("repeat", args=1)
 def _repeat(app, value):
-    repeat = app.window.repeat
+    po = app.player_options
     if value in ["0", "off"]:
-        repeat.set_active(False)
+        po.repeat = False
     elif value in ["1", "on"]:
-        repeat.set_active(True)
+        print_d("Enabling repeat")
+        po.repeat = True
     elif value in ["t", "toggle"]:
-        repeat.set_active(not repeat.get_active())
+        po.repeat = not po.repeat
 
 
 @registry.register("seek", args=1)
@@ -217,11 +213,7 @@ def _seek(app, time):
 
 @registry.register("play-file", args=1)
 def _play_file(app, value):
-    filename = os.path.realpath(value)
-    song = app.library.add_filename(filename, add=False)
-    if song:
-        if app.player.go_to(song):
-            app.player.paused = False
+    app.window.open_file(value)
 
 
 @registry.register("toggle-window")
@@ -299,7 +291,7 @@ def _filter(app, value):
 @registry.register("query", args=1)
 def _query(app, value):
     if app.browser.can_filter_text():
-        app.browser.filter_text(value.decode("utf-8", "replace"))
+        app.browser.filter_text(fsn2text(value))
 
 
 @registry.register("unfilter")
@@ -351,7 +343,7 @@ def _enqueue_files(app, value):
     songs = []
     for param in split_escape(value, ","):
         try:
-            song_path = uri_to_path(param)
+            song_path = uri2fsn(param)
         except ValueError:
             song_path = param
         if song_path in library:
@@ -382,7 +374,6 @@ def _quit(app):
 @registry.register("status")
 def _status(app):
     player = app.player
-    window = app.window
     f = StringIO()
 
     if player.paused:
@@ -390,9 +381,10 @@ def _status(app):
     else:
         strings = ["playing"]
     strings.append(type(app.browser).__name__)
+    po = app.player_options
     strings.append("%0.3f" % player.volume)
-    strings.append(window.order.get_active_name())
-    strings.append((window.repeat.get_active() and "on") or "off")
+    strings.append("shuffle" if po.shuffle else "inorder")
+    strings.append("on" if po.repeat else "off")
     progress = 0
     if player.info:
         length = player.info.get("~#length", 0)
