@@ -34,11 +34,13 @@ import traceback
 
 from gi.repository import Gtk, Pango, Gdk, GLib
 
+from quodlibet import _
 from quodlibet import const
 from quodlibet.qltk import Icons
 from quodlibet.compat import exec_
 from quodlibet.plugins.songsmenu import SongsMenuPlugin
 from quodlibet.util.collection import Collection
+from quodlibet.util import print_
 
 
 class PyConsole(SongsMenuPlugin):
