@@ -14,25 +14,27 @@ import os
 import shutil
 import time
 
+from senf import fsn2uri, fsnative, fsn2text, devnull, bytes2fsn, path2fsn
+
+from quodlibet import _
 from quodlibet import util
 from quodlibet import config
-from quodlibet.util.path import mkdir, fsdecode, mtime, expanduser, is_fsnative
-from quodlibet.util.path import normalize_path, fsnative, escape_filename
+from quodlibet.util.path import mkdir, mtime, expanduser, \
+    normalize_path, escape_filename, ismount
 from quodlibet.util.string import encode, decode, isascii
 
 from quodlibet.util import iso639
-from quodlibet.util.uri import URI
 from quodlibet.util import human_sort_key as human, capitalize
 
 from quodlibet.util.tags import TAG_ROLES, TAG_TO_SORT
-from quodlibet.compat import iteritems, string_types, text_type, number_types
+from quodlibet.compat import iteritems, string_types, text_type, \
+    number_types, listitems, izip_longest, integer_types, PY3, listfilter
 
 from ._image import ImageContainer
+from ._misc import AudioFileError, translate_errors
 
-try:
-    from itertools import izip_longest
-except ImportError:  # python3.x
-    izip = zip
+
+translate_errors
 
 MIGRATE = {"~#playcount", "~#laststarted", "~#lastplayed", "~#added",
            "~#skipcount", "~#rating", "~bookmark"}
@@ -42,11 +44,19 @@ PEOPLE = ["artist", "albumartist", "author", "composer", "~performers",
           "originalartist", "lyricist", "arranger", "conductor"]
 """Sources of the ~people tag, most important first"""
 
-INTERN_NUM_DEFAULT = {"~#lastplayed", "~#laststarted", "~#playcount",
-                      "~#skipcount", "~#length", "~#bitrate", "~#filesize"}
-"""Default to 0"""
+TIME_TAGS = {"~#lastplayed", "~#laststarted", "~#added", "~#mtime"}
+"""Time in seconds since epoch, defaults to 0"""
 
-FILESYSTEM_TAGS = {"~filename", "~basename", "~dirname"}
+SIZE_TAGS = {"~#filesize"}
+"""Size in bytes, defaults to 0"""
+
+NUMERIC_ZERO_DEFAULT = {"~#skipcount", "~#playcount", "~#length", "~#bitrate"}
+"""Defaults to 0"""
+
+NUMERIC_ZERO_DEFAULT.update(TIME_TAGS)
+NUMERIC_ZERO_DEFAULT.update(SIZE_TAGS)
+
+FILESYSTEM_TAGS = {"~filename", "~basename", "~dirname", "~mountpoint"}
 """Values are bytes in Linux instead of unicode"""
 
 SORT_TO_TAG = dict([(v, k) for (k, v) in iteritems(TAG_TO_SORT)])
@@ -71,8 +81,8 @@ def decode_value(tag, value):
     elif isinstance(value, float):
         return u"%.2f" % value
     elif tag in FILESYSTEM_TAGS:
-        return fsdecode(value)
-    return unicode(value)
+        return fsn2text(value)
+    return text_type(value)
 
 
 class AudioFile(dict, ImageContainer):
@@ -117,7 +127,7 @@ class AudioFile(dict, ImageContainer):
     """MIME types this class can represent"""
 
     def __song_key(self):
-        return (self("~#disc"), self("~#track"),
+        return (self("~#disc", 1), self("~#track", 1),
             human(self("artistsort")),
             self.get("musicbrainz_artistid", ""),
             human(self.get("title", "")),
@@ -146,7 +156,7 @@ class AudioFile(dict, ImageContainer):
         elif tag == "artistsort":
             return artist_sort
         elif tag in FILESYSTEM_TAGS:
-            return lambda song: fsdecode(song(tag), note=False)
+            return lambda song: fsn2text(song(tag))
         elif tag.startswith("~#") and "~" not in tag[2:]:
             return lambda song: song(tag)
         return lambda song: human(song(tag))
@@ -160,15 +170,33 @@ class AudioFile(dict, ImageContainer):
         pass
 
     def __setitem__(self, key, value):
-        if not self.__dict__:
-            # unpickle case.. we can't fail
-            dict.__setitem__(self, key, value)
-            return
+        # validate key
+        if PY3:
+            if not isinstance(key, text_type):
+                raise TypeError("key has to be str")
+        else:
+            if isinstance(key, text_type):
+                # we try to save keys as encoded ASCII to save memory
+                # under PY2. Everything else besides ASCII combined with
+                # unicode breaks hashing even if the default encoding
+                # it utf-8.
+                try:
+                    key = key.encode("ascii")
+                except UnicodeEncodeError:
+                    pass
+            elif isinstance(key, bytes):
+                # make sure we set ascii keys only
+                key.decode("ascii")
+            else:
+                raise TypeError("key needs to be unicode or ASCII str")
 
+        # validate value
         if key.startswith("~#"):
-            assert isinstance(value, number_types)
+            if not isinstance(value, number_types):
+                raise TypeError
         elif key in FILESYSTEM_TAGS:
-            assert is_fsnative(value)
+            if not isinstance(value, fsnative):
+                value = path2fsn(value)
         else:
             value = text_type(value)
 
@@ -180,8 +208,7 @@ class AudioFile(dict, ImageContainer):
 
     def __delitem__(self, key):
         dict.__delitem__(self, key)
-        if not self.__dict__:
-            return
+
         pop = self.__dict__.pop
         pop("album_key", None)
         pop("sort_key", None)
@@ -212,9 +239,14 @@ class AudioFile(dict, ImageContainer):
         return self is not other
 
     def reload(self):
-        """Reload an audio file from disk. The caller is responsible for
-        handling any errors."""
+        """Reload an audio file from disk. If reloading fails nothing will
+        change.
 
+        Raises:
+            AudioFileError: if the file fails to load
+        """
+
+        backup = dict(self)
         fn = self["~filename"]
         saved = {}
         for key in self:
@@ -222,14 +254,19 @@ class AudioFile(dict, ImageContainer):
                 saved[key] = self[key]
         self.clear()
         self["~filename"] = fn
-        self.__init__(fn)
-        self.update(saved)
+        try:
+            self.__init__(fn)
+        except AudioFileError:
+            self.update(backup)
+            raise
+        else:
+            self.update(saved)
 
     def realkeys(self):
         """Returns a list of keys that are not internal, i.e. they don't
         have '~' in them."""
 
-        return filter(lambda s: s[:1] != "~", self.keys())
+        return listfilter(lambda s: s[:1] != "~", self.keys())
 
     def prefixkeys(self, prefix):
         """Returns a list of dict keys that either match prefix or start
@@ -333,9 +370,9 @@ class AudioFile(dict, ImageContainer):
                 try:
                     return self["~uri"]
                 except KeyError:
-                    return URI.frompath(self["~filename"])
+                    return fsn2uri(self["~filename"])
             elif key == "format":
-                return self.get("~format", self.format)
+                return self.get("~format", text_type(self.format))
             elif key == "codec":
                 codec = self.get("~codec")
                 if codec is None:
@@ -415,8 +452,8 @@ class AudioFile(dict, ImageContainer):
                 key = "~" + key
                 if key in self:
                     return self[key]
-                elif key in INTERN_NUM_DEFAULT:
-                    return dict.get(self, key, 0)
+                elif key in NUMERIC_ZERO_DEFAULT:
+                    return 0
                 else:
                     try:
                         val = self[key[2:]]
@@ -521,13 +558,14 @@ class AudioFile(dict, ImageContainer):
         """
 
         if "~" in key or key == "title":
-            v = self(key, u"")
             if key in FILESYSTEM_TAGS:
-                v = fsdecode(v)
+                v = fsn2text(self(key, fsnative()))
+            else:
+                v = self(key, u"")
         else:
             v = self.get(key, u"")
 
-        if isinstance(v, (int, long, float)):
+        if isinstance(v, number_types):
             return v
         else:
             return v.replace("\n", ", ")
@@ -637,7 +675,7 @@ class AudioFile(dict, ImageContainer):
     def mounted(self):
         """Return true if the disk the file is on is mounted, or
         the file is not on a disk."""
-        return os.path.ismount(self.get("~mountpoint", "/"))
+        return ismount(self.get("~mountpoint", "/"))
 
     def can_multiple_values(self, key=None):
         """If no arguments are given, return a list of tags that can
@@ -687,45 +725,19 @@ class AudioFile(dict, ImageContainer):
 
         self.sanitize(newname)
 
-    def website(self):
-        """Look for a URL in the audio metadata, or a Google search
-        if no URL can be found."""
-
-        if "website" in self:
-            return self.list("website")[0]
-        for cont in self.list("contact") + self.list("comment"):
-            c = cont.lower()
-            if (c.startswith("http://") or c.startswith("https://") or
-                    c.startswith("www.")):
-                return cont
-            elif c.startswith("//www."):
-                return "http:" + cont
-        else:
-            text = "http://www.google.com/search?q="
-            esc = lambda c: ord(c) > 127 and '%%%x' % ord(c) or c
-            if "labelid" in self:
-                text += ''.join(map(esc, self["labelid"]))
-            else:
-                artist = util.escape("+".join(self("artist").split()))
-                album = util.escape("+".join(self("album").split()))
-                artist = encode(artist)
-                album = encode(album)
-                artist = "%22" + ''.join(map(esc, artist)) + "%22"
-                album = "%22" + ''.join(map(esc, album)) + "%22"
-                text += artist + "+" + album
-            text += "&ie=UTF8"
-            return text
-
     def sanitize(self, filename=None):
         """Fill in metadata defaults. Find ~mountpoint, ~#mtime, ~#filesize
-        and ~#added. Check for null bytes in tags."""
+        and ~#added. Check for null bytes in tags.
+
+        Does not raise.
+        """
 
         # Replace nulls with newlines, trimming zero-length segments
-        for key, val in self.items():
+        for key, val in listitems(self):
             if isinstance(val, string_types) and '\0' in val:
                 self[key] = '\n'.join(filter(lambda s: s, val.split('\0')))
             # Remove unnecessary defaults
-            if key in INTERN_NUM_DEFAULT and val == 0:
+            if key in NUMERIC_ZERO_DEFAULT and val == 0:
                 del self[key]
 
         if filename:
@@ -733,7 +745,7 @@ class AudioFile(dict, ImageContainer):
         elif "~filename" not in self:
             raise ValueError("Unknown filename!")
 
-        assert is_fsnative(self["~filename"])
+        assert isinstance(self["~filename"], fsnative)
 
         if self.is_file:
             self["~filename"] = normalize_path(
@@ -744,8 +756,8 @@ class AudioFile(dict, ImageContainer):
                 head, tail = os.path.split(head)
                 # Prevent infinite loop without a fully-qualified filename
                 # (the unit tests use these).
-                head = head or "/"
-                if os.path.ismount(head):
+                head = head or fsnative(u"/")
+                if ismount(head):
                     self["~mountpoint"] = head
         else:
             self["~mountpoint"] = fsnative(u"/")
@@ -772,7 +784,11 @@ class AudioFile(dict, ImageContainer):
             self["~#mtime"] = 0
 
     def to_dump(self):
-        """A string of 'key=value' lines, similar to vorbiscomment output."""
+        """A string of 'key=value' lines, similar to vorbiscomment output.
+
+        Returns:
+            bytes
+        """
 
         def encode_key(k):
             return encode(k) if isinstance(k, text_type) else k
@@ -780,45 +796,58 @@ class AudioFile(dict, ImageContainer):
         s = []
         for k in self.keys():
             enc_key = encode_key(k)
+            assert isinstance(enc_key, bytes)
 
-            if isinstance(self[k], int) or isinstance(self[k], long):
-                s.append("%s=%d" % (enc_key, self[k]))
+            if isinstance(self[k], integer_types):
+                l = enc_key + encode("=%d" % self[k])
+                s.append(l)
             elif isinstance(self[k], float):
-                s.append("%s=%f" % (enc_key, self[k]))
+                l = enc_key + encode("=%f" % self[k])
+                s.append(l)
             else:
                 for v2 in self.list(k):
-                    if isinstance(v2, str):
-                        s.append("%s=%s" % (enc_key, v2))
-                    else:
-                        s.append("%s=%s" % (enc_key, encode(v2)))
-        for k in (INTERN_NUM_DEFAULT - set(self.keys())):
+                    if not isinstance(v2, bytes):
+                        v2 = encode(v2)
+                    s.append(enc_key + b"=" + v2)
+        for k in (NUMERIC_ZERO_DEFAULT - set(self.keys())):
             enc_key = encode_key(k)
-            s.append("%s=%d" % (enc_key, self.get(k, 0)))
+            l = enc_key + encode("=%d" % self.get(k, 0))
+            s.append(l)
         if "~#rating" not in self:
-            s.append("~#rating=%f" % self("~#rating"))
-        s.append("~format=%s" % self.format)
-        s.append("")
-        return "\n".join(s)
+            s.append(encode("~#rating=%f" % self("~#rating")))
+        s.append(encode("~format=%s" % self.format))
+        s.append(b"")
+        return b"\n".join(s)
 
     def from_dump(self, text):
-        """Parses the text created with to_dump and adds the found tags."""
+        """Parses the text created with to_dump and adds the found tags.
+
+        Args:
+            text (bytes)
+        """
 
         def decode_key(key):
             """str if ascii, otherwise decode using utf-8"""
+
+            if PY3:
+                return decode(key)
+
             try:
                 key.decode("ascii")
             except ValueError:
                 return decode(key)
             return key
 
-        for line in text.split("\n"):
+        for line in text.split(b"\n"):
             if not line:
                 continue
-            parts = line.split("=")
-            key = parts[0]
-            val = "=".join(parts[1:])
+            parts = line.split(b"=")
+            key = decode_key(parts[0])
+            val = b"=".join(parts[1:])
             if key == "~format":
                 pass
+            elif key in FILESYSTEM_TAGS:
+                self.add(key, bytes2fsn(val, "utf-8"))
             elif key.startswith("~#"):
                 try:
                     self.add(key, int(val))
@@ -828,7 +857,7 @@ class AudioFile(dict, ImageContainer):
                     except ValueError:
                         pass
             else:
-                self.add(decode_key(key), decode(val))
+                self.add(key, decode(val))
 
     def change(self, key, old_value, new_value):
         """Change 'old_value' to 'new_value' for the given metadata key.
@@ -900,7 +929,12 @@ class AudioFile(dict, ImageContainer):
             return min(15, scale)
 
     def write(self):
-        """Write metadata back to the file."""
+        """Write metadata back to the file.
+
+        Raises:
+            AudioFileError: in case writing fails
+        """
+
         raise NotImplementedError
 
     @property
@@ -951,7 +985,7 @@ class AudioFile(dict, ImageContainer):
 
 # Looks like the real thing.
 DUMMY_SONG = AudioFile({
-    '~#length': 234, '~filename': '/dev/null',
+    '~#length': 234, '~filename': devnull,
     'artist': 'The Artist', 'album': 'An Example Album',
     'title': 'First Track', 'tracknumber': 1,
     'date': '2010-12-31',
