@@ -7,6 +7,7 @@
 
 from mutagen.wavpack import WavPack
 
+from ._audio import translate_errors
 from ._apev2 import APEv2File
 
 
@@ -15,11 +16,12 @@ class WavpackFile(APEv2File):
     mimes = ["audio/x-wavpack"]
 
     def __init__(self, filename):
-        audio = WavPack(filename)
+        with translate_errors():
+            audio = WavPack(filename)
         super(WavpackFile, self).__init__(filename, audio)
         self["~#length"] = audio.info.length
         self.sanitize(filename)
 
-info = WavpackFile
+loader = WavpackFile
 types = [WavpackFile]
 extensions = [".wv"]
