@@ -15,12 +15,16 @@ import shutil
 import subprocess
 import tempfile
 
+from senf import fsn2text
+
+from quodlibet import _
 from quodlibet import util
 from quodlibet.formats import EmbeddedImage, AudioFileError
-from quodlibet.util.path import mtime, fsdecode
+from quodlibet.util.path import mtime
 from quodlibet.pattern import Pattern, error as PatternError
 from quodlibet.util.tags import USER_TAGS, sortkey
 from quodlibet.util.tagsfrompath import TagsFromPattern
+from quodlibet.compat import text_type, iteritems
 
 from .base import Command, CommandError
 from .util import print_terse_table, copy_mtime, list_tags, print_table, \
@@ -55,7 +59,7 @@ class ListCommand(Command):
         if not options.columns:
             order = nicks
         else:
-            order = map(str.strip, options.columns.split(","))
+            order = [n.strip() for n in options.columns.split(",")]
 
         song = self.load_song(path)
         tags = list_tags(song, machine=options.all, terse=options.terse)
@@ -89,7 +93,7 @@ class TagsCommand(Command):
         if not options.columns:
             order = nicks
         else:
-            order = map(str.strip, options.columns.split(","))
+            order = [n.strip() for n in options.columns.split(",")]
 
         tags = []
         for key in USER_TAGS:
@@ -164,13 +168,13 @@ class EditCommand(Command):
             u"",
             u"#" * 80,
             u"# Lines that are empty or start with '#' will be ignored",
-            u"# File: %r" % fsdecode(song("~filename")),
+            u"# File: %r" % fsn2text(song("~filename")),
         ]
 
         return u"\n".join(lines)
 
     def _text_to_song(self, text, song):
-        assert isinstance(text, unicode)
+        assert isinstance(text, text_type)
 
         # parse
         tags = {}
@@ -197,7 +201,7 @@ class EditCommand(Command):
                     self.log("Add %s=%s" % (key, value))
                     song.add(key, value)
 
-        for key, values in tags.iteritems():
+        for key, values in iteritems(tags):
             if not song.can_change(key):
                 raise CommandError(
                     "Can't change key '%(key-name)s'." % {"key-name": key})
@@ -237,10 +241,10 @@ class EditCommand(Command):
             try:
                 subprocess.check_call(editor_args + [path])
             except subprocess.CalledProcessError as e:
-                self.log(unicode(e))
+                self.log(text_type(e))
                 raise CommandError(_("Editing aborted"))
             except OSError as e:
-                self.log(unicode(e))
+                self.log(text_type(e))
                 raise CommandError(
                     _("Starting text editor '%(editor-name)s' failed.") % {
                         "editor-name": editor_args[0]})
@@ -282,8 +286,8 @@ class SetCommand(Command):
         if len(args) < 3:
             raise CommandError(_("Not enough arguments"))
 
-        tag = args[0]
-        value = args[1].decode("utf-8")
+        tag = fsn2text(args[0])
+        value = fsn2text(args[1])
         paths = args[2:]
 
         songs = []
@@ -422,8 +426,8 @@ class AddCommand(Command):
         if len(args) < 3:
             raise CommandError(_("Not enough arguments"))
 
-        tag = args[0]
-        value = args[1].decode("utf-8")
+        tag = fsn2text(args[0])
+        value = fsn2text(args[1])
         paths = args[2:]
 
         songs = []
@@ -468,20 +472,20 @@ class InfoCommand(Command):
         if not options.columns:
             order = nicks
         else:
-            order = map(str.strip, options.columns.split(","))
+            order = [n.strip() for n in options.columns.split(",")]
 
         if not options.terse:
             tags = []
             for key in ["~format", "~codec", "~encoding", "~length",
                         "~bitrate", "~filesize"]:
-                tags.append((util.tag(key), unicode(song.comma(key))))
+                tags.append((util.tag(key), text_type(song.comma(key))))
 
             print_table(tags, headers, nicks, order)
         else:
             tags = []
             for key in ["~format", "~codec", "~encoding", "~#length",
                         "~#bitrate", "~#filesize"]:
-                tags.append((key.lstrip("#~"), unicode(song(key))))
+                tags.append((key.lstrip("#~"), text_type(song(key))))
 
             print_terse_table(tags, nicks, order)
 
@@ -686,7 +690,7 @@ class FillCommand(Command):
         rows = []
         for song in songs:
             match = pattern.match(song)
-            row = [fsdecode(song("~basename"))]
+            row = [fsn2text(song("~basename"))]
             for header in pattern.headers:
                 row.append(match.get(header, u""))
             rows.append(row)
