@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
-# Copyright 2004-2012 Joe Wreschnig, Michael Urman, Iñigo Serna, Nick Boultbee
+# Copyright 2004-2012 Joe Wreschnig, Michael Urman, Iñigo Serna
+#           2011-2016 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -9,13 +10,14 @@ import sys
 
 from gi.repository import Gtk, Pango, Gdk
 
+from quodlibet import C_, _
 from quodlibet import qltk
-
 from quodlibet import config
 from quodlibet import util
 
 from quodlibet.util import massagers
 
+from quodlibet.formats import AudioFileError
 from quodlibet.qltk.completion import LibraryValueCompletion
 from quodlibet.qltk.tagscombobox import TagsComboBox, TagsComboBoxEntry
 from quodlibet.qltk.views import RCMHintedTreeView, TreeViewColumn
@@ -28,10 +30,13 @@ from quodlibet.qltk._editutils import EditingPluginHandler, OverwriteWarning
 from quodlibet.qltk._editutils import WriteFailedError
 from quodlibet.qltk import Icons
 from quodlibet.plugins import PluginManager
-from quodlibet.util import connect_obj
+from quodlibet.util import connect_obj, gdecode
+from quodlibet.util.i18n import numeric_phrase
 from quodlibet.util.tags import USER_TAGS, MACHINE_TAGS, sortkey as tagsortkey
 from quodlibet.util.string.splitters import (split_value, split_title,
     split_people, split_album)
+from quodlibet.compat import iteritems, string_types, text_type, listkeys, \
+    listmap, itervalues
 
 
 class Comment(object):
@@ -55,20 +60,20 @@ class Comment(object):
 
     def _paren(self):
         if self.shared:
-            return ngettext('missing from %d song',
-                            'missing from %d songs',
-                            self.missing) % self.missing
+            return numeric_phrase('missing from %d song',
+                                  'missing from %d songs',
+                                  self.missing)
         elif self.complete:
-            return ngettext('different across %d song',
-                            'different across %d songs',
-                            self.total) % self.total
+            return numeric_phrase('different across %d song',
+                                  'different across %d songs',
+                                  self.total)
         else:
-            d = ngettext('different across %d song',
-                          'different across %d songs',
-                          self.have) % self.have
-            m = ngettext('missing from %d song',
-                          'missing from %d songs',
-                          self.missing) % self.missing
+            d = numeric_phrase('different across %d song',
+                               'different across %d songs',
+                               self.have)
+            m = numeric_phrase('missing from %d song',
+                               'missing from %d songs',
+                               self.missing)
             return ", ".join([d, m])
 
     def is_special(self):
@@ -146,10 +151,10 @@ class AudioFileGroup(dict):
         self._can_change = can_change
 
         # collect comment representations
-        for tag, count in keys.iteritems():
+        for tag, count in iteritems(keys):
             first_value = first[tag]
-            if not isinstance(first_value, basestring):
-                first_value = unicode(first_value)
+            if not isinstance(first_value, string_types):
+                first_value = text_type(first_value)
             shared = all[tag]
             complete = count == total
             if shared and complete:
@@ -188,13 +193,11 @@ class SplitValues(Gtk.ImageMenuItem):
             label=_("Split into _Multiple Values"), use_underline=True)
         self.set_image(Gtk.Image.new_from_icon_name(
             Icons.EDIT_FIND_REPLACE, Gtk.IconSize.MENU))
-        spls = config.get("editing", "split_on").decode(
-            'utf-8', 'replace').split()
+        spls = config.gettext("editing", "split_on").split()
         self.set_sensitive(len(split_value(value, spls)) > 1)
 
     def activated(self, tag, value):
-        spls = config.get("editing", "split_on").decode(
-            'utf-8', 'replace').split()
+        spls = config.gettext("editing", "split_on").split()
         return [(tag, v) for v in split_value(value, spls)]
 
 
@@ -225,13 +228,11 @@ class SplitTitle(Gtk.ImageMenuItem):
             label=_("Split _Version out of Title"), use_underline=True)
         self.set_image(Gtk.Image.new_from_icon_name(
             Icons.EDIT_FIND_REPLACE, Gtk.IconSize.MENU))
-        spls = config.get("editing", "split_on").decode(
-            'utf-8', 'replace').split()
+        spls = config.gettext("editing", "split_on").split()
         self.set_sensitive(bool(split_title(value, spls)[1]))
 
     def activated(self, tag, value):
-        spls = config.get("editing", "split_on").decode(
-            'utf-8', 'replace').split()
+        spls = config.gettext("editing", "split_on").split()
         title, versions = split_title(value, spls)
         return [(tag, title)] + [("version", v) for v in versions]
 
@@ -244,13 +245,11 @@ class SplitPerson(Gtk.ImageMenuItem):
         super(SplitPerson, self).__init__(label=self.title, use_underline=True)
         self.set_image(Gtk.Image.new_from_icon_name(
             Icons.EDIT_FIND_REPLACE, Gtk.IconSize.MENU))
-        spls = config.get("editing", "split_on").decode(
-            'utf-8', 'replace').split()
+        spls = config.gettext("editing", "split_on").split()
         self.set_sensitive(bool(split_people(value, spls)[1]))
 
     def activated(self, tag, value):
-        spls = config.get("editing", "split_on").decode(
-            'utf-8', 'replace').split()
+        spls = config.gettext("editing", "split_on").split()
         artist, others = split_people(value, spls)
         return [(tag, artist)] + [(self.needs[0], o) for o in others]
 
@@ -350,23 +349,19 @@ class AddTagDialog(Dialog):
             return self.__tag.tag
 
     def get_value(self):
-        return self.__val.get_text().decode("utf-8")
+        return gdecode(self.__val.get_text())
 
     def __validate(self, editable, add, invalid, box):
         tag = self.get_tag()
         value = self.get_value()
-        fmt = massagers.tags.get(tag)
-        if fmt:
-            valid = fmt.is_valid(value)
-        else:
-            valid = True
+        valid = massagers.is_valid(tag, value)
         add.set_sensitive(valid)
         if valid:
             invalid.hide()
             box.set_tooltip_text("")
         else:
             invalid.show()
-            box.set_tooltip_text(fmt.error)
+            box.set_tooltip_text(massagers.error_message(tag, value))
 
     def run(self):
         self.show()
@@ -414,23 +409,27 @@ class EditTags(Gtk.VBox):
         self._view = view
         selection = view.get_selection()
         render = Gtk.CellRendererPixbuf()
-        column = TreeViewColumn(_("Write"), render)
+        column = TreeViewColumn()
+        column.pack_start(render, True)
+        column.set_fixed_width(24)
+        column.set_expand(False)
 
         def cdf_write(col, rend, model, iter_, *args):
             entry = model.get_value(iter_)
+            rend.set_property('sensitive', entry.edited or entry.deleted)
             if entry.canedit or entry.deleted:
-                rend.set_property('sensitive', entry.edited or entry.deleted)
                 if entry.deleted:
                     rend.set_property('icon-name', Icons.EDIT_DELETE)
                 else:
                     rend.set_property('icon-name', Icons.EDIT)
             else:
-                rend.set_property('icon-name', Icons.DIALOG_PASSWORD)
+                rend.set_property('icon-name', Icons.CHANGES_PREVENT)
         column.set_cell_data_func(render, cdf_write)
         view.append_column(column)
 
         render = Gtk.CellRendererText()
-        column = TreeViewColumn(_('Tag'), render)
+        column = TreeViewColumn(title=_('Tag'))
+        column.pack_start(render, True)
 
         def cell_data_tag(column, cell, model, iter_, data):
             entry = model.get_value(iter_)
@@ -452,7 +451,8 @@ class EditTags(Gtk.VBox):
         render.connect('edited', self.__edit_tag, model)
         render.connect(
             'editing-started', self.__value_editing_started, model, library)
-        column = TreeViewColumn(_('Value'), render)
+        column = TreeViewColumn(title=_('Value'))
+        column.pack_start(render, True)
 
         def cell_data_value(column, cell, model, iter_, data):
             entry = model.get_value(iter_)
@@ -560,7 +560,8 @@ class EditTags(Gtk.VBox):
         for b in buttons:
             b.set_sensitive(True)
 
-    def __paste(self, clip, text, (rend, path)):
+    def __paste(self, clip, text, args):
+        rend, path = args
         if text:
             rend.emit('edited', path, text.strip())
 
@@ -620,7 +621,8 @@ class EditTags(Gtk.VBox):
                 else:
                     b.connect('activate', self.__menu_activate, view)
 
-                    if (not min(map(self.__songinfo.can_change, b.needs) + [1])
+                    if (not min(listmap(self.__songinfo.can_change, b.needs) +
+                                [1])
                             or comment.is_special()):
                         b.set_sensitive(False)
 
@@ -651,7 +653,7 @@ class EditTags(Gtk.VBox):
         remove.set_sensitive(bool(rows))
 
     def __add_new_tag(self, model, tag, value):
-        assert isinstance(value, unicode)
+        assert isinstance(value, text_type)
         iters = [i for (i, v) in model.iterrows() if v.tag == tag]
         if iters and not self.__songinfo.can_multiple_values(tag):
             title = _("Unable to add tag")
@@ -680,9 +682,9 @@ class EditTags(Gtk.VBox):
                 break
             tag = add.get_tag()
             value = add.get_value()
-            assert isinstance(value, unicode)
-            if tag in massagers.tags:
-                value = massagers.tags[tag].validate(value)
+            assert isinstance(value, text_type)
+            value = massagers.validate(tag, value)
+            assert isinstance(value, text_type)
             if not self.__songinfo.can_change(tag):
                 title = _("Invalid tag")
                 msg = _("Invalid tag <b>%s</b>\n\nThe files currently"
@@ -714,7 +716,7 @@ class EditTags(Gtk.VBox):
         added = {}
         renamed = {}
 
-        for entry in model.itervalues():
+        for entry in itervalues(model):
             if entry.edited and not (entry.deleted or entry.renamed):
                 if entry.origvalue is not None:
                     l = updated.setdefault(entry.tag, [])
@@ -747,7 +749,7 @@ class EditTags(Gtk.VBox):
                     break
 
             changed = False
-            for key, values in updated.iteritems():
+            for key, values in iteritems(updated):
                 for (new_value, old_value) in values:
                     if song.can_change(key):
                         if old_value is None:
@@ -756,13 +758,13 @@ class EditTags(Gtk.VBox):
                             song.change(key, old_value.text, new_value.text)
                         changed = True
 
-            for key, values in added.iteritems():
+            for key, values in iteritems(added):
                 for value in values:
                     if song.can_change(key):
                         song.add(key, value.text)
                         changed = True
 
-            for key, values in deleted.iteritems():
+            for key, values in iteritems(deleted):
                 for value in values:
                     if not value.shared:
                         # In case it isn't shared we don't know the actual
@@ -776,7 +778,7 @@ class EditTags(Gtk.VBox):
                         changed = True
 
             save_rename = []
-            for new_tag, values in renamed.iteritems():
+            for new_tag, values in iteritems(renamed):
                 for old_tag, new_value, old_value in values:
                     if (song.can_change(new_tag) and old_tag in song):
                         if not new_value.is_special():
@@ -797,7 +799,7 @@ class EditTags(Gtk.VBox):
             if changed:
                 try:
                     song.write()
-                except:
+                except AudioFileError:
                     util.print_exc()
                     WriteFailedError(self, song).run()
                     library.reload(song, changed=was_changed)
@@ -815,21 +817,20 @@ class EditTags(Gtk.VBox):
             b.set_sensitive(not all_done)
 
     def __edit_tag(self, renderer, path, new_value, model):
-        new_value = new_value.decode("utf-8")
+        new_value = gdecode(new_value)
         new_value = ', '.join(new_value.splitlines())
         path = Gtk.TreePath.new_from_string(path)
         entry = model[path][0]
         error_dialog = None
 
-        if entry.tag in massagers.tags:
-            fmt = massagers.tags[entry.tag]
-            if not fmt.is_valid(new_value):
-                error_dialog = qltk.WarningMessage(
-                    self, _("Invalid value"),
-                    _("Invalid value: <b>%(value)s</b>\n\n%(error)s") % {
-                    "value": new_value, "error": fmt.error})
-            else:
-                new_value = fmt.validate(new_value)
+        if not massagers.is_valid(entry.tag, new_value):
+            error_dialog = qltk.WarningMessage(
+                self, _("Invalid value"),
+                _("Invalid value: <b>%(value)s</b>\n\n%(error)s") % {
+                "value": new_value,
+                "error": massagers.error_message(entry.tag, new_value)})
+        else:
+            new_value = massagers.validate(entry.tag, new_value)
 
         comment = entry.value
         changed = comment.text != new_value
@@ -862,15 +863,14 @@ class EditTags(Gtk.VBox):
             # validate one value and never write it back..
 
             text = entry.value.text
-            if new_tag in massagers.tags:
-                fmt = massagers.tags[new_tag]
-                if not fmt.is_valid(text):
-                    qltk.WarningMessage(
-                        self, _("Invalid value"),
-                        _("Invalid value: <b>%(value)s</b>\n\n%(error)s") % {
-                          "value": text, "error": fmt.error}).run()
-                    return
-                text = fmt.validate(text)
+            if not massagers.is_valid(new_tag, text):
+                qltk.WarningMessage(
+                    self, _("Invalid value"),
+                    _("Invalid value: <b>%(value)s</b>\n\n%(error)s") % {
+                      "value": text,
+                      "error": massagers.error_message(new_tag, text)}).run()
+                return
+            text = massagers.validate(new_tag, text)
 
             if entry.origvalue is None:
                 # The tag hasn't been saved yet, so we can just update
@@ -901,19 +901,7 @@ class EditTags(Gtk.VBox):
         except TypeError:
             return Gdk.EVENT_PROPAGATE
 
-        if event.button == Gdk.BUTTON_PRIMARY and col is view.get_columns()[0]:
-            model = view.get_model()
-            row = model[path]
-            entry = row[0]
-            # In case we have a (partially) shared value, write it
-            # to all songs. For unshared/incomplete do nothing
-            if entry.value.shared:
-                entry.edited = not entry.edited
-                if entry.edited:
-                    entry.value.complete = True
-            model.row_changed(row.path, row.iter)
-            return Gdk.EVENT_STOP
-        elif event.button == Gdk.BUTTON_MIDDLE and \
+        if event.button == Gdk.BUTTON_MIDDLE and \
                 col == view.get_columns()[2]:
             display = Gdk.DisplayManager.get().get_default_display()
             selection = Gdk.SELECTION_PRIMARY
@@ -938,7 +926,7 @@ class EditTags(Gtk.VBox):
             self.__songinfo = AudioFileGroup(songs)
         songinfo = self.__songinfo
 
-        keys = songinfo.keys()
+        keys = listkeys(songinfo)
         default_tags = get_default_tags()
         keys = set(keys + default_tags)
 
