@@ -10,6 +10,7 @@
 import os
 
 from gi.repository import Gtk, Gdk, GLib, Gio, GObject
+from senf import uri2fsn, fsnative
 
 import quodlibet
 
@@ -20,7 +21,9 @@ from quodlibet import formats
 from quodlibet import qltk
 from quodlibet import util
 from quodlibet import app
+from quodlibet import _
 
+from quodlibet.qltk.appwindow import AppWindow
 from quodlibet.update import UpdateDialog
 from quodlibet.formats.remote import RemoteFile
 from quodlibet.qltk.browser import LibraryBrowser, FilterMenu
@@ -34,7 +37,8 @@ from quodlibet.qltk.info import SongInfo
 from quodlibet.qltk.information import Information
 from quodlibet.qltk.msg import ErrorMessage, WarningMessage
 from quodlibet.qltk.notif import StatusBar, TaskController
-from quodlibet.qltk.playorder import PlayOrder
+from quodlibet.qltk.playorder import PlayOrderWidget, RepeatSongForever, \
+    RepeatListForever
 from quodlibet.qltk.pluginwin import PluginWindow
 from quodlibet.qltk.properties import SongProperties
 from quodlibet.qltk.prefs import PreferencesWindow
@@ -42,14 +46,14 @@ from quodlibet.qltk.queue import QueueExpander
 from quodlibet.qltk.songlist import SongList, get_columns, set_columns
 from quodlibet.qltk.songmodel import PlaylistMux
 from quodlibet.qltk.x import RVPaned, Align, ScrolledWindow, Action
-from quodlibet.qltk.x import SymbolicIconImage, ToggleAction, RadioAction
+from quodlibet.qltk.x import ToggleAction, RadioAction
 from quodlibet.qltk.x import SeparatorMenuItem, MenuItem, CellRendererPixbuf
 from quodlibet.qltk import Icons
 from quodlibet.qltk.about import AboutDialog
 from quodlibet.util import copool, connect_destroy, connect_after_destroy
 from quodlibet.util.library import get_scan_dirs, set_scan_dirs
 from quodlibet.util import connect_obj, print_d
-from quodlibet.util.path import glib2fsnative, get_home_dir, uri_to_path
+from quodlibet.util.path import glib2fsn, get_home_dir
 from quodlibet.util.library import background_filter, scan_library
 from quodlibet.qltk.window import PersistentWindowMixin, Window, on_first_map
 from quodlibet.qltk.songlistcolumns import SongListColumn
@@ -67,7 +71,7 @@ class PlayerOptions(GObject.Object):
     """
 
     __gproperties__ = {
-        'random': (bool, '', '', False,
+        'shuffle': (bool, '', '', False,
                    GObject.ParamFlags.READABLE | GObject.ParamFlags.WRITABLE),
         'repeat': (bool, '', '', False,
                    GObject.ParamFlags.READABLE | GObject.ParamFlags.WRITABLE),
@@ -79,24 +83,20 @@ class PlayerOptions(GObject.Object):
     }
 
     def __init__(self, window):
-        """windows is a QuodLibetWindow"""
+        """`window` is a QuodLibetWindow"""
 
         super(PlayerOptions, self).__init__()
 
-        self._repeat = window.repeat
-        self._rid = self._repeat.connect(
-            "toggled", lambda *x: self.notify("repeat"))
-
         self._stop_after = window.stop_after
         self._said = self._stop_after.connect(
             "toggled", lambda *x: self.notify("stop-after"))
 
         def order_changed(*args):
-            self.notify("random")
+            self.notify("shuffle")
             self.notify("single")
 
-        self._order = window.order
-        self._oid = self._order.connect("changed", order_changed)
+        self._order_widget = window.order
+        self._oid = self._order_widget.connect("changed", order_changed)
 
         window.connect("destroy", self._window_destroy)
 
@@ -104,12 +104,9 @@ class PlayerOptions(GObject.Object):
         self.destroy()
 
     def destroy(self):
-        if self._repeat:
-            self._repeat.disconnect(self._rid)
-            self._repeat = None
-        if self._order:
-            self._order.disconnect(self._oid)
-            self._order = None
+        if self._order_widget:
+            self._order_widget.disconnect(self._oid)
+            self._order_widget = None
         if self._stop_after:
             self._stop_after.disconnect(self._said)
             self._stop_after = None
@@ -128,34 +125,38 @@ class PlayerOptions(GObject.Object):
         When `repeat` is True the current song will be replayed.
         """
 
-        return self._order.get_active_name() == "onesong"
+        return (self._order_widget.repeated and
+                self._order_widget.repeater is RepeatSongForever)
 
     @single.setter
     def single(self, value):
-        if value and not self.single:
-            self._order.set_active_by_name("onesong")
-        elif not value and self.single:
-            self._order.set_active_by_name("inorder")
+        if value:
+            self.repeat = True
+            self._order_widget.repeater = RepeatSongForever
+        else:
+            self.repeat = False
+            self._order_widget.repeater = RepeatListForever
 
     @property
-    def random(self):
-        """If a random based play order is active"""
+    def shuffle(self):
+        """If a shuffle-like (reordering) play order is active"""
 
-        return self._order.get_shuffle()
+        return self._order_widget.shuffled
 
-    @random.setter
-    def random(self, value):
-        self._order.set_shuffle(value)
+    @shuffle.setter
+    def shuffle(self, value):
+        self._order_widget.shuffled = value
 
     @property
     def repeat(self):
-        """If the playlist will be restarted if it ended"""
+        """If the player is in some kind of repeat mode"""
 
-        return self._repeat.get_active()
+        return self._order_widget.repeated
 
     @repeat.setter
     def repeat(self, value):
-        self._repeat.set_active(value)
+        print_d("setting repeated to %s" % value)
+        self._order_widget.repeated = value
 
     @property
     def stop_after(self):
@@ -367,43 +368,6 @@ class TopBar(Gtk.Toolbar):
 
     def __song_art_changed(self, player, songs, library):
         self.image.refresh()
-        refresh_albums = []
-        for song in songs:
-            # Album browser only (currently):
-            album = library.albums.get(song.album_key, None)
-            if album:
-                album.scan_cover(force=True)
-                refresh_albums.append(album)
-        if refresh_albums:
-            library.albums.refresh(refresh_albums)
-
-
-class RepeatButton(Gtk.ToggleButton):
-
-    def __init__(self):
-        super(RepeatButton, self).__init__(
-            image=SymbolicIconImage(
-                "media-playlist-repeat", Gtk.IconSize.SMALL_TOOLBAR))
-
-        self.set_name("ql-repeat-button")
-        qltk.add_css(self, """
-            #ql-repeat-button {
-                padding: 0px;
-            }
-        """)
-        self.set_size_request(26, 26)
-
-        self.set_tooltip_text(_("Restart the playlist when finished"))
-
-        self.bind_config("settings", "repeat")
-
-    def bind_config(self, section, option):
-        self.set_active(config.getboolean(section, option))
-
-        def toggled_cb(*args):
-            config.set(section, option, self.get_active())
-
-        self.connect('toggled', toggled_cb)
 
 
 class QueueButton(Gtk.ToggleButton):
@@ -430,17 +394,9 @@ class QueueButton(Gtk.ToggleButton):
 
 class StatusBarBox(Gtk.HBox):
 
-    def __init__(self, model, player, queue):
+    def __init__(self, play_order, queue):
         super(StatusBarBox, self).__init__(spacing=6)
-
-        self.order = order = PlayOrder(model, player)
-        self.pack_start(order, False, True, 0)
-
-        self.repeat = repeat = RepeatButton()
-        self.pack_start(repeat, False, True, 0)
-        repeat.connect('toggled', self.__repeat, model)
-        model.repeat = repeat.get_active()
-
+        self.pack_start(play_order, False, True, 0)
         self.statusbar = StatusBar(TaskController.default_instance)
         self.pack_start(self.statusbar, True, True, 0)
         queue_button = QueueButton()
@@ -450,9 +406,6 @@ class StatusBarBox(Gtk.HBox):
 
         self.pack_start(queue_button, False, True, 0)
 
-    def __repeat(self, button, model):
-        model.repeat = button.get_active()
-
 
 class AppMenu(object):
     """Implements a app menu proxy mirroring some main menu items
@@ -712,7 +665,7 @@ class SongListPaned(RVPaned):
             config.set(section, option, str(self.get_relative()))
 
 
-class QuodLibetWindow(Window, PersistentWindowMixin):
+class QuodLibetWindow(Window, PersistentWindowMixin, AppWindow):
 
     def __init__(self, library, player, headless=False, restore_cb=None):
         super(QuodLibetWindow, self).__init__(dialog=False)
@@ -720,11 +673,12 @@ class QuodLibetWindow(Window, PersistentWindowMixin):
 
         self.__destroyed = False
         self.__update_title(player)
-        self.set_default_size(550, 450)
+        self.set_default_size(600, 480)
 
         main_box = Gtk.VBox()
         self.add(main_box)
 
+        self.__player = player
         # create main menubar, load/restore accelerator groups
         self.__library = library
         ui = self.__create_menu(player, library)
@@ -792,9 +746,9 @@ class QuodLibetWindow(Window, PersistentWindowMixin):
         self.__browserbox = Align(bottom=3)
         main_box.pack_start(self.__browserbox, True, True, 0)
 
-        statusbox = StatusBarBox(self.songlist.model, player, self.qexpander)
-        self.order = statusbox.order
-        self.repeat = statusbox.repeat
+        play_order = PlayOrderWidget(self.songlist.model, player)
+        statusbox = StatusBarBox(play_order, self.qexpander)
+        self.order = play_order
         self.statusbar = statusbox.statusbar
 
         main_box.pack_start(
@@ -847,7 +801,7 @@ class QuodLibetWindow(Window, PersistentWindowMixin):
         self.songlist.connect('popup-menu', self.__songs_popup_menu)
         self.songlist.connect('columns-changed', self.__cols_changed)
         self.songlist.connect('columns-changed', self.__hide_headers)
-        self.songlist.info.connect("changed", self.__set_time)
+        self.songlist.info.connect("changed", self.__set_totals)
 
         lib = library.librarian
         connect_destroy(lib, 'changed', self.__song_changed, player)
@@ -898,9 +852,20 @@ class QuodLibetWindow(Window, PersistentWindowMixin):
         quit_item = self.ui.get_widget('/Menu/File/Quit')
         quit_item.hide()
 
-    def get_osx_is_persistent(self):
+    def get_is_persistent(self):
         return True
 
+    def open_file(self, filename):
+        assert isinstance(filename, fsnative)
+
+        song = self.__library.add_filename(filename, add=False)
+        if song is not None:
+            if self.__player.go_to(song):
+                self.__player.paused = False
+            return True
+        else:
+            return False
+
     def __player_error(self, player, song, player_error):
         # it's modal, but mmkeys etc. can still trigger new ones
         if self._playback_error_dialog:
@@ -965,7 +930,7 @@ class QuodLibetWindow(Window, PersistentWindowMixin):
         error = False
         for uri in uris:
             try:
-                filename = uri_to_path(uri)
+                filename = uri2fsn(uri)
             except ValueError:
                 filename = None
 
@@ -1094,7 +1059,7 @@ class QuodLibetWindow(Window, PersistentWindowMixin):
 
         act = Action(name="Shortcuts", label=_("_Keyboard Shortcuts"))
         act.connect('activate', self.__keyboard_shortcuts)
-        ag.add_action_with_accel(act, "<Primary>F1")
+        ag.add_action_with_accel(act, "<Primary>question")
 
         act = Action(name="About", label=_("_About"),
                      icon_name=Icons.HELP_ABOUT)
@@ -1426,7 +1391,7 @@ class QuodLibetWindow(Window, PersistentWindowMixin):
             fns, do_watch = dialog.run()
             dialog.destroy()
             if fns:
-                fns = map(glib2fsnative, fns)
+                fns = map(glib2fsn, fns)
                 # scan them
                 self.last_dir = fns[0]
                 copool.add(self.__library.scan, fns, cofuncid="library",
@@ -1444,7 +1409,7 @@ class QuodLibetWindow(Window, PersistentWindowMixin):
             fns = dialog.run()
             dialog.destroy()
             if fns:
-                fns = map(glib2fsnative, fns)
+                fns = map(glib2fsn, fns)
                 self.last_dir = os.path.dirname(fns[0])
                 for filename in map(os.path.realpath, fns):
                     self.__library.add_filename(filename)
@@ -1526,8 +1491,8 @@ class QuodLibetWindow(Window, PersistentWindowMixin):
             self.browser.filter_text(query.encode('utf-8'))
             self.browser.activate()
 
-    def __set_time(self, info, songs):
+    def __set_totals(self, info, songs):
         length = sum(song.get("~#length", 0) for song in songs)
         t = self.browser.status_text(count=len(songs),
-                                     time=util.format_time_long(length))
+                                     time=util.format_time_preferred(length))
         self.statusbar.set_default_text(t)
