@@ -9,10 +9,12 @@ import os
 import unicodedata
 
 from gi.repository import Gtk, Gdk
+from senf import fsn2text, text2fsn
 
 import quodlibet
 from quodlibet import qltk
 from quodlibet import util
+from quodlibet import _
 
 from quodlibet.plugins import PluginManager
 from quodlibet.pattern import FileFromPattern
@@ -24,8 +26,8 @@ from quodlibet.qltk.models import ObjectStore
 from quodlibet.qltk import Icons, Button
 from quodlibet.qltk.wlw import WritingWindow
 from quodlibet.util import connect_obj, gdecode
-from quodlibet.util.path import fsdecode, fsnative
 from quodlibet.util.path import strip_win32_incompat_from_path
+from quodlibet.compat import itervalues
 
 
 NBP = os.path.join(quodlibet.get_user_dir(), "lists", "renamepatterns")
@@ -73,8 +75,7 @@ class StripDiacriticals(FilterCheckButton):
     _order = 1.2
 
     def filter(self, original, filename):
-        filename = fsdecode(filename)
-        return fsnative(filter(lambda s: not unicodedata.combining(s),
+        return u"".join(filter(lambda s: not unicodedata.combining(s),
                                unicodedata.normalize('NFKD', filename)))
 
 
@@ -85,9 +86,7 @@ class StripNonASCII(FilterCheckButton):
     _order = 1.3
 
     def filter(self, original, filename):
-        filename = fsdecode(filename)
-        return fsnative(
-            u"".join(map(lambda s: (s <= "~" and s) or u"_", filename)))
+        return u"".join(map(lambda s: (s <= "~" and s) or u"_", filename))
 
 
 class Lowercase(FilterCheckButton):
@@ -115,7 +114,7 @@ class Entry(object):
 
     @property
     def name(self):
-        return fsdecode(self.song("~basename"))
+        return fsn2text(self.song("~basename"))
 
 
 class RenameFiles(Gtk.VBox):
@@ -236,7 +235,7 @@ class RenameFiles(Gtk.VBox):
         skip_all = False
         self.view.freeze_child_notify()
 
-        for entry in model.itervalues():
+        for entry in itervalues(model):
             song = entry.song
             new_name = entry.new_name
             old_name = entry.name
@@ -244,7 +243,7 @@ class RenameFiles(Gtk.VBox):
                 continue
 
             try:
-                library.rename(song, fsnative(new_name), changed=was_changed)
+                library.rename(song, text2fsn(new_name), changed=was_changed)
             except Exception:
                 util.print_exc()
                 if skip_all:
@@ -284,7 +283,7 @@ class RenameFiles(Gtk.VBox):
     def __preview(self, songs):
         model = self.view.get_model()
         if songs is None:
-            songs = [e.song for e in model.itervalues()]
+            songs = [e.song for e in itervalues(model)]
 
         pattern_text = gdecode(self.combo.get_child().get_text())
 
@@ -306,7 +305,7 @@ class RenameFiles(Gtk.VBox):
 
         # native paths
         orignames = [song["~filename"] for song in songs]
-        newnames = [pattern.format(song) for song in songs]
+        newnames = [fsn2text(pattern.format(song)) for song in songs]
         for f in self.filter_box.filters:
             if f.active:
                 newnames = f.filter_list(orignames, newnames)
@@ -314,7 +313,7 @@ class RenameFiles(Gtk.VBox):
         model.clear()
         for song, newname in zip(songs, newnames):
             entry = Entry(song)
-            entry.new_name = fsdecode(newname)
+            entry.new_name = newname
             model.append(row=[entry])
 
         self.preview.set_sensitive(False)
