@@ -9,12 +9,14 @@
 # published by the Free Software Foundation
 
 from gi.repository import Gtk, GLib, Gdk, GObject
+from senf import uri2fsn
 
 from quodlibet import app
 from quodlibet import config
 from quodlibet import const
 from quodlibet import qltk
 from quodlibet import util
+from quodlibet import _
 
 from quodlibet.query import Query
 from quodlibet.pattern import Pattern
@@ -29,13 +31,13 @@ from quodlibet.formats._audio import TAG_TO_SORT, AudioFile
 from quodlibet.qltk.x import SeparatorMenuItem
 from quodlibet.qltk.songlistcolumns import create_songlist_column
 from quodlibet.util import connect_destroy
-from quodlibet.util.path import uri_to_path
+from quodlibet.compat import xrange, string_types, iteritems, listfilter
 
 
 DND_QL, DND_URI_LIST = range(2)
 
 
-class SongInfoSelection(GObject.Object):
+class SongSelectionInfo(GObject.Object):
     """
     Songs which get included in the status bar summary.
 
@@ -64,7 +66,7 @@ class SongInfoSelection(GObject.Object):
     }
 
     def __init__(self, songlist):
-        super(SongInfoSelection, self).__init__()
+        super(SongSelectionInfo, self).__init__()
 
         self.__idle = None
         self.__songlist = songlist
@@ -78,6 +80,11 @@ class SongInfoSelection(GObject.Object):
         if self.__idle:
             GLib.source_remove(self.__idle)
 
+    def refresh(self):
+        songlist = self.__songlist
+        songs = songlist.get_selected_songs() or songlist.get_songs()
+        self.emit('changed', songs)
+
     def _update_songs(self, songs):
         """After making changes (filling the list) call this to
         skip any queued changes and emit the passed songs instead"""
@@ -157,9 +164,9 @@ def get_sort_tag(tag):
         tag = "album"
 
     if "<" in tag:
-        for key, value in replace_order.iteritems():
+        for key, value in iteritems(replace_order):
             tag = tag.replace("<%s>" % key, "<%s>" % value)
-        for key, value in TAG_TO_SORT.iteritems():
+        for key, value in iteritems(TAG_TO_SORT):
             tag = tag.replace("<%s>" % key,
                                "<{1}|<{1}>|<{0}>>".format(key, value))
         tag = Pattern(tag).format
@@ -193,11 +200,19 @@ class SongListDnDMixin(object):
     """DnD support for the SongList class"""
 
     def setup_drop(self, library):
+        self.connect('drag-begin', self.__drag_begin)
         self.connect('drag-motion', self.__drag_motion)
         self.connect('drag-leave', self.__drag_leave)
         self.connect('drag-data-get', self.__drag_data_get)
         self.connect('drag-data-received', self.__drag_data_received, library)
 
+    def __drag_begin(self, *args):
+        ok, state = Gtk.get_current_event_state()
+        if ok and state & qltk.get_primary_accel_mod():
+            self.__force_copy = True
+        else:
+            self.__force_copy = False
+
     def enable_drop(self, by_row=True):
         targets = [
             ("text/x-quodlibet-songs", Gtk.TargetFlags.SAME_APP, DND_QL),
@@ -210,6 +225,7 @@ class SongListDnDMixin(object):
         self.drag_dest_set(Gtk.DestDefaults.ALL, targets,
                            Gdk.DragAction.COPY | Gdk.DragAction.MOVE)
         self.__drop_by_row = by_row
+        self.__force_copy = False
 
     def disable_drop(self):
         targets = [
@@ -225,7 +241,8 @@ class SongListDnDMixin(object):
         if self.__drop_by_row:
             self.set_drag_dest(x, y)
             self.scroll_motion(x, y)
-            if Gtk.drag_get_source_widget(ctx) == self:
+            if Gtk.drag_get_source_widget(ctx) == self and \
+                    not self.__force_copy:
                 kind = Gdk.DragAction.MOVE
             else:
                 kind = Gdk.DragAction.COPY
@@ -267,15 +284,15 @@ class SongListDnDMixin(object):
         model = view.get_model()
         if info == DND_QL:
             filenames = qltk.selection_get_filenames(sel)
-            move = (Gtk.drag_get_source_widget(ctx) == view)
+            move = bool(ctx.get_selected_action() & Gdk.DragAction.MOVE)
         elif info == DND_URI_LIST:
             def to_filename(s):
                 try:
-                    return uri_to_path(s)
+                    return uri2fsn(s)
                 except ValueError:
                     return None
 
-            filenames = filter(None, map(to_filename, sel.get_uris()))
+            filenames = listfilter(None, map(to_filename, sel.get_uris()))
             move = False
         else:
             Gtk.drag_finish(ctx, False, False, etime)
@@ -288,7 +305,7 @@ class SongListDnDMixin(object):
             elif filename not in library:
                 to_add.append(library.librarian[filename])
         library.add(to_add)
-        songs = filter(None, map(library.get, filenames))
+        songs = listfilter(None, map(library.get, filenames))
         if not songs:
             Gtk.drag_finish(ctx, bool(not filenames), False, etime)
             return
@@ -357,7 +374,7 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
             # Translators: The substituted string is the name of the
             # selected column (a translated tag name).
             b = qltk.MenuItem(
-                _("_Filter on %s") % util.tag(t, True), Icons.EDIT_SELECT_ALL)
+                _("_Filter on %s") % util.tag(t, True), Icons.EDIT_FIND)
             b.connect('activate', self.__filter_on, t, songs, browser)
             return b
 
@@ -380,7 +397,7 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
         super(SongList, self).__init__()
         self._register_instance(SongList)
         self.set_model(model_cls())
-        self.info = SongInfoSelection(self)
+        self.info = SongSelectionInfo(self)
         self.set_size_request(200, 150)
         self.set_rules_hint(True)
         self.get_selection().set_mode(Gtk.SelectionMode.MULTIPLE)
@@ -562,7 +579,7 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
     def __search_func(self, model, column, key, iter, *args):
         for column in self.get_columns():
             value = model.get_value(iter)(column.header_name)
-            if not isinstance(value, basestring):
+            if not isinstance(value, string_types):
                 continue
             elif key in value.lower() or key in value:
                 return False
@@ -1015,8 +1032,8 @@ class SongList(AllTreeView, SongListDnDMixin, DragScroll,
             column.set_reorderable(True)
             self.append_column(column)
 
-        self.columns_autosize()
         self.set_sort_orders(old_sort)
+        self.columns_autosize()
 
         self.handler_unblock(self.__csig)
 
