@@ -12,14 +12,18 @@ import time
 import datetime
 
 from gi.repository import Gtk, Pango, GLib
+from senf import fsnative, fsn2text
 
+from quodlibet import _
 from quodlibet import util
 from quodlibet import config
 from quodlibet.pattern import Pattern
 from quodlibet.qltk.views import TreeViewColumnButton
 from quodlibet.qltk import add_css
-from quodlibet.util.path import fsdecode, unexpand, fsnative
+from quodlibet.util.path import unexpand
 from quodlibet.formats._audio import FILESYSTEM_TAGS
+from quodlibet.compat import text_type, string_types, PY2, listvalues, \
+    listitems
 
 
 def create_songlist_column(t):
@@ -281,8 +285,9 @@ class DateColumn(WideTextColumn):
             else:
                 format_ = "%x"
             stamp = time.localtime(stamp)
-            encoding = util.get_locale_encoding()
-            text = time.strftime(format_, stamp).decode(encoding)
+            text = time.strftime(format_, stamp)
+            if PY2:
+                text = text.decode(util.get_locale_encoding())
             cell.set_property('text', text)
 
 
@@ -312,7 +317,7 @@ class FSColumn(WideTextColumn):
         return values[0] if values else fsnative(u"")
 
     def _apply_value(self, model, iter_, cell, value):
-        cell.set_property('text', fsdecode(unexpand(value)))
+        cell.set_property('text', fsn2text(unexpand(value)))
 
 
 class PatternColumn(WideTextColumn):
@@ -372,7 +377,7 @@ class NumericColumn(TextColumn):
         if isinstance(value, float):
             text = u"%.2f" % round(value, 2)
         else:
-            text = unicode(value)
+            text = text_type(value)
 
         cell.set_property('text', text)
         self._recalc_width(model.get_path(iter_), text)
@@ -391,15 +396,15 @@ class NumericColumn(TextColumn):
         end = end[0]
 
         # compute the cell width for all drawn cells in range +/- 3
-        for key, value in self._texts.items():
+        for key, value in listitems(self._texts):
             if not (start - 3) <= key <= (end + 3):
                 del self._texts[key]
-            elif isinstance(value, basestring):
+            elif isinstance(value, string_types):
                 self._texts[key] = self._cell_width(value)
 
         # resize if too small or way too big and above the minimum
         width = self.get_width()
-        needed_width = max([self._get_min_width()] + self._texts.values())
+        needed_width = max([self._get_min_width()] + listvalues(self._texts))
         if width < needed_width:
             self._resize(needed_width)
         elif width - needed_width >= self._cell_width("0"):
