@@ -6,10 +6,11 @@
 # published by the Free Software Foundation
 
 from gi.repository import Gtk
+from senf import fsn2text
 
 from quodlibet import qltk
 from quodlibet import util
-
+from quodlibet import _
 from quodlibet.formats import AudioFileError
 from quodlibet.qltk._editutils import OverwriteWarning, WriteFailedError
 from quodlibet.qltk.views import HintedTreeView, TreeViewColumn
@@ -17,8 +18,8 @@ from quodlibet.qltk.wlw import WritingWindow
 from quodlibet.qltk.x import Button
 from quodlibet.qltk.models import ObjectStore
 from quodlibet.qltk import Icons
-from quodlibet.util.path import fsdecode
 from quodlibet.util import connect_obj, gdecode
+from quodlibet.compat import text_type, itervalues
 
 
 class Entry(object):
@@ -29,7 +30,7 @@ class Entry(object):
 
     @property
     def name(self):
-        return fsdecode(self.song("~basename"))
+        return fsn2text(self.song("~basename"))
 
 
 class TrackNumbers(Gtk.VBox):
@@ -150,7 +151,7 @@ class TrackNumbers(Gtk.VBox):
         win = WritingWindow(parent, len(model))
         was_changed = set()
         all_done = False
-        for entry in model.itervalues():
+        for entry in itervalues(model):
             song, track = entry.song, entry.tracknumber
             if song.get("tracknumber") == track:
                 win.step()
@@ -188,7 +189,7 @@ class TrackNumbers(Gtk.VBox):
             if total:
                 s = u"%d/%d" % (row.path.get_indices()[0] + start, total)
             else:
-                s = unicode(row.path.get_indices()[0] + start)
+                s = text_type(row.path.get_indices()[0] + start)
             entry = row[0]
             entry.tracknumber = s
             model.row_changed(row.path, row.iter)
@@ -198,7 +199,7 @@ class TrackNumbers(Gtk.VBox):
 
     def __update(self, songs, total, model, save, revert):
         if songs is None:
-            songs = [e.song for e in model.itervalues()]
+            songs = [e.song for e in itervalues(model)]
         else:
             songs = list(songs)
 
