@@ -29,7 +29,7 @@ import unicodedata
 import sys
 
 from quodlibet.util import re_escape
-from quodlibet.compat import iteritems
+from quodlibet.compat import iteritems, urlopen, text_type, xrange, unichr
 
 
 _DIACRITIC_CACHE = {
@@ -133,7 +133,7 @@ _DIACRITIC_CACHE = {
 }
 
 # See misc/uca_decomps.py
-_UCA_DECOMPS = {
+_UCA_DECOMPS_CACHE = {
     u'AA': u'\ua732',
     u'AE': u'\xc6\u01e2\u01fc',
     u'AO': u'\ua734',
@@ -224,6 +224,48 @@ _UCA_DECOMPS = {
 }
 
 
+def get_decomps_mapping(regenerate=False):
+    """This takes the decomps.txt file of the Unicode UCA and gives us a cases
+    where a letter can be decomposed for collation and that mapping isn't in
+    NFKD.
+    """
+
+    if not regenerate:
+        return _UCA_DECOMPS_CACHE
+
+    mapping = {}
+
+    h = urlopen("http://unicode.org/Public/UCA/8.0.0/decomps.txt")
+    for line in h.read().splitlines():
+        if line.startswith("#"):
+            continue
+
+        to_uni = lambda x: unichr(int(x, 16))
+        is_letter = lambda x: unicodedata.category(x) in ("Lu", "Ll", "Lt")
+
+        cp, line = line.split(";", 1)
+        tag, line = line.split(";", 1)
+        decomp, line = line.split("#", 1)
+        decomp = map(to_uni, decomp.strip().split())
+        cp = to_uni(cp)
+
+        if not is_letter(cp):
+            continue
+
+        decomp = filter(is_letter, decomp)
+        simple = "".join(decomp)
+        if not simple:
+            continue
+
+        # skip anything we get from normalization
+        if unicodedata.normalize("NFKD", cp)[0] == simple:
+            continue
+
+        mapping[simple] = mapping.get(simple, "") + cp
+
+    return mapping
+
+
 def diacritic_for_letters(regenerate=False):
     """Returns a mapping for combining diacritic mark to ascii characters
     for which they can be used to combine to a single unicode char.
@@ -338,6 +380,7 @@ def _construct_regexp(pattern, mapping):
     literals = []
 
     for op, av in pattern:
+        op = str(op).lower()
 
         if literals and op != "literal":
             parts.append(_fixup_literal_list(literals, mapping))
@@ -349,6 +392,7 @@ def _construct_regexp(pattern, mapping):
             literals.append(av)
             continue
         elif op == "category":
+            av = str(av).lower()
             cats = {
                 "category_word": u"\\w",
                 "category_not_word": u"\\W",
@@ -369,6 +413,7 @@ def _construct_regexp(pattern, mapping):
             in_parts = []
             for entry in av:
                 op, eav = entry
+                op = str(op).lower()
                 if op == "literal":
                     in_parts.append(_fixup_literal(eav, True, mapping))
                 else:
@@ -391,6 +436,7 @@ def _construct_regexp(pattern, mapping):
             if op == "min_repeat":
                 parts[-1] = parts[-1] + u"?"
         elif op == "at":
+            av = str(av).lower()
             ats = {
                 "at_beginning": u"^",
                 "at_end": u"$",
@@ -445,7 +491,7 @@ def _construct_regexp(pattern, mapping):
 def re_replace_literals(text, mapping):
     """Raises NotImplementedError or re.error"""
 
-    assert isinstance(text, unicode)
+    assert isinstance(text, text_type)
 
     pattern = sre_parse.parse(text)
     return _construct_regexp(pattern, mapping)
@@ -455,7 +501,7 @@ def re_replace_literals(text, mapping):
 _mapping = generate_re_mapping(diacritic_for_letters(regenerate=False))
 
 # add more from the UCA decomp dataset
-for cp, repl in iteritems(_UCA_DECOMPS):
+for cp, repl in iteritems(get_decomps_mapping(regenerate=False)):
     _mapping[cp] = _mapping.get(cp, u"") + repl
 
 
@@ -472,6 +518,6 @@ def re_add_variants(text):
     case something is not supported NotImplementedError gets raised.
     """
 
-    assert isinstance(text, unicode)
+    assert isinstance(text, text_type)
 
     return re_replace_literals(text, _mapping)
