@@ -11,8 +11,9 @@
 import time
 import operator
 
-from quodlibet.compat import floordiv
-from quodlibet.util.path import fsdecode
+from senf import fsn2text, fsnative
+
+from quodlibet.compat import floordiv, text_type
 from quodlibet.util import parse_date
 from quodlibet.plugins.query import QUERY_HANDLER
 from quodlibet.plugins.query import QueryPluginError
@@ -33,7 +34,7 @@ class Node(object):
         raise NotImplementedError
 
     def filter(self, sequence):
-        return filter(self.search, sequence)
+        return [s for s in sequence if self.search(s)]
 
     def _unpack(self):
         return self
@@ -227,11 +228,7 @@ class NumexprTag(Numexpr):
     """Numeric tag"""
 
     def __init__(self, tag):
-        if isinstance(tag, unicode):
-            self._tag = tag.encode("utf-8")
-        else:
-            self._tag = tag
-
+        self._tag = tag
         self._ftag = "~#" + self._tag
 
     def evaluate(self, data, time, use_date):
@@ -318,7 +315,10 @@ class NumexprBinary(Numexpr):
         val = self.__expr.evaluate(data, time, use_date)
         val2 = self.__expr2.evaluate(data, time, use_date)
         if val is not None and val2 is not None:
-            return self.__op(val, val2)
+            try:
+                return self.__op(val, val2)
+            except ZeroDivisionError:
+                return val * float('inf')
         return None
 
     def __repr__(self):
@@ -471,24 +471,25 @@ class Tag(Node):
                 self._names.append(name)
 
     def search(self, data):
+        fs_default = fsnative()
+
         for name in self._names:
             val = data.get(name)
             if val is None:
-                # filename is the only real entry that's a path
-                if name == "filename":
-                    val = fsdecode(data.get("~filename", ""))
+                if name in ("filename", "mountpoint"):
+                    val = fsn2text(data.get("~" + name, fs_default))
                 else:
                     val = data.get("~" + name, "")
 
-            if self.res.search(unicode(val)):
+            if self.res.search(text_type(val)):
                 return True
 
         for name in self.__intern:
-            if self.res.search(unicode(data(name))):
+            if self.res.search(text_type(data(name))):
                 return True
 
         for name in self.__fs:
-            if self.res.search(fsdecode(data(name))):
+            if self.res.search(fsn2text(data(name, fs_default))):
                 return True
 
         return False
