@@ -6,13 +6,16 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
+from quodlibet import _
 from quodlibet.util.dprint import print_d
 from quodlibet.formats import AudioFileError
 from quodlibet import util
 from quodlibet import qltk
 from quodlibet.qltk.wlw import WritingWindow
+from quodlibet.util.misc import total_ordering
 
 
+@total_ordering
 class SongWrapper(object):
     __slots__ = ['_song', '_updated', '_needs_write']
 
@@ -52,11 +55,11 @@ class SongWrapper(object):
         else:
             return super(SongWrapper, self).__setattr__(attr, value)
 
-    def __cmp__(self, other):
-        try:
-            return cmp(self._song, other._song)
-        except:
-            return cmp(self._song, other)
+    def __eq__(self, other):
+        return self._song == other._song
+
+    def __lt__(self, other):
+        return self._song < other._song
 
     def __getitem__(self, *args):
         return self._song.__getitem__(*args)
@@ -88,11 +91,11 @@ def ListWrapper(songs):
             return None
         else:
             return SongWrapper(song)
-    return map(wrap, songs)
+    return [wrap(s) for s in songs]
 
 
 def check_wrapper_changed(library, parent, songs):
-    needs_write = filter(lambda s: s._needs_write, songs)
+    needs_write = list(filter(lambda s: s._needs_write, songs))
 
     if needs_write:
         win = WritingWindow(parent, len(needs_write))
